@@ -1,6 +1,6 @@
 /*
 * this file is part of the oxygen gtk engine
-* Copyright (c) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
+* Copyright (c) 2010 Hugo Pereira Da Costa <hugo.pereira@free.fr>
 * Copyright (c) 2010 Ruslan Kabatsayev <b7.10110111@gmail.com>
 *
 * This  library is free  software; you can  redistribute it and/or
@@ -28,6 +28,8 @@
 #include "oxygentimeline.h"
 #include "config.h"
 
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 
 #include <algorithm>
@@ -56,6 +58,7 @@ namespace Oxygen
         _kdeFallbackIconTheme( "gnome" ),
         _inactiveChangeSelectionColor( false ),
         _useIconEffect( true ),
+        _useBackgroundGradient( true ),
         _checkBoxStyle( CS_CHECK ),
         _tabStyle( TS_SINGLE ),
         _scrollBarAddLineButtons( 2 ),
@@ -67,9 +70,11 @@ namespace Oxygen
         _viewDrawTreeBranchLines( true ),
         _viewDrawTriangularExpander( true ),
         _viewTriangularExpanderSize( ArrowSmall ),
+        _viewInvertSortIndicator( false ),
         _menuHighlightMode( MM_DARK ),
         _windowDragEnabled( true ),
         _windowDragMode( WD_FULL ),
+        _useWMMoveResize( true ),
         _startDragDist( 4 ),
         _startDragTime( 500 ),
         _animationsEnabled( true ),
@@ -88,6 +93,7 @@ namespace Oxygen
         _windecoBlendType( FollowStyleHint ),
         _activeShadowConfiguration( Palette::Active ),
         _inactiveShadowConfiguration( Palette::Inactive ),
+        _backgroundOpacity( 255 ),
         _argbEnabled( true ),
         _initialized( false ),
         _kdeColorsInitialized( false ),
@@ -96,28 +102,16 @@ namespace Oxygen
     {}
 
     //_________________________________________________________
-    void QtSettings::loadKdeGlobals( void )
-    {
-
-        _kdeGlobals.clear();
-        for( PathList::const_reverse_iterator iter = _kdeConfigPathList.rbegin(); iter != _kdeConfigPathList.rend(); ++iter )
-        { _kdeGlobals.merge( OptionMap( sanitizePath( *iter + "/kdeglobals" ) ) ); }
-
-        #if OXYGEN_DEBUG
-        std::cerr << "Oxygen::QtSettings::loadKdeGlobals - kdeglobals: " << std::endl;
-        std::cerr << _kdeGlobals << std::endl;
-        #endif
-
-    }
-
-    //_________________________________________________________
-    void QtSettings::initialize( unsigned int flags )
+    bool QtSettings::initialize( unsigned int flags )
     {
 
         const bool forced( flags&Forced );
 
-        if( _initialized && !forced ) return;
-        _initialized = true;
+        // no attempt at initializing if gtk settings is not yet set
+        if( !gtk_settings_get_default() ) return false;
+
+        if( _initialized && !forced ) return false;
+        else if( !forced ) _initialized = true;
 
         if( g_getenv( "KDE_FULL_SESSION" ) )
         { _KDESession = true; }
@@ -130,16 +124,37 @@ namespace Oxygen
             initArgb();
         }
 
-
-        // dialog button ordering
-        GtkSettings* settings( gtk_settings_get_default() );
-        gtk_settings_set_long_property( settings, "gtk-alternative-button-order", 1, "oxygen-gtk" );
+        // keep track of whats changed
+        bool changed( false );
 
         // configuration path
-        _kdeConfigPathList = kdeConfigPathList();
+        {
+            const PathList old( _kdeConfigPathList );
+            _kdeConfigPathList = kdeConfigPathList();
+            changed |= (old != _kdeConfigPathList );
+        }
 
-        // load kdeglobals
-        loadKdeGlobals();
+        // icon path
+        {
+            const PathList old( _kdeIconPathList );
+            _kdeIconPathList = kdeIconPathList();
+            changed |= (old != _kdeIconPathList );
+        }
+
+        // load kdeglobals and oxygen option maps
+        const bool kdeGlobalsChanged = loadKdeGlobals();
+        const bool oxygenChanged = loadOxygen();
+
+        // do nothing if settings not changed
+        if( forced && !(changed||kdeGlobalsChanged||oxygenChanged) ) return false;
+
+        // dialog button ordering
+        /* TODO: in principle this should be needed only once */
+        if( flags & Extra )
+        {
+            GtkSettings* settings( gtk_settings_get_default() );
+            gtk_settings_set_long_property( settings, "gtk-alternative-button-order", 1, "oxygen-gtk" );
+        }
 
         // clear gtkrc
         _rc.clear();
@@ -165,10 +180,7 @@ namespace Oxygen
             // reload icons
             #if OXYGEN_ICON_HACK
             if( flags & Icons )
-            {
-                _kdeIconPathList = kdeIconPathList();
-                loadKdeIcons();
-            }
+            { loadKdeIcons(); }
             #endif
 
         }
@@ -180,23 +192,9 @@ namespace Oxygen
             generateGtkColors();
         }
 
-        // deal with pathbar button margins
-        // this needs to be done programatically in order to properly account for RTL locales
-        _rc.addSection( "oxygen-pathbutton-internal", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( "  GtkButton::inner-border = { 2, 2, 1, 0 }" );
-
-        if( gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL )
-        {
-
-            _rc.addToCurrentSection( "  GtkToggleButton::inner-border={ 10, 0, 1, 0 }" );
-
-        } else {
-
-            _rc.addToCurrentSection( "  GtkToggleButton::inner-border={ 0, 10, 1, 0 }" );
-
-        }
-
-        _rc.matchWidgetClassToSection( "*PathBar.GtkToggleButton", "oxygen-pathbutton-internal" );
+        // apply extra programatically set metrics metrics
+        if( flags & Extra )
+        { loadExtraOptions(); }
 
         // print generated Gtkrc and commit
         #if OXYGEN_DEBUG
@@ -211,7 +209,63 @@ namespace Oxygen
         std::cerr << "Oxygen::QtSettings::initialize - done. " << std::endl;
         #endif
 
-        return;
+        return true;
+
+    }
+
+    //_________________________________________________________
+    bool QtSettings::runCommand( const std::string& command, char*& result ) const
+    { return g_spawn_command_line_sync( command.c_str(), &result, 0L, 0L, 0L ) && result; }
+
+    //_________________________________________________________
+    bool QtSettings::loadKdeGlobals( void )
+    {
+
+        // save backup
+        OptionMap old = _kdeGlobals;
+
+        // clear and reload
+        _kdeGlobals.clear();
+        for( PathList::const_reverse_iterator iter = _kdeConfigPathList.rbegin(); iter != _kdeConfigPathList.rend(); ++iter )
+        {
+            const std::string filename( sanitizePath( *iter + "/kdeglobals" ) );
+            _kdeGlobals.merge( OptionMap( filename ) );
+            monitorFile( filename );
+        }
+
+        #if OXYGEN_DEBUG
+        std::cerr << "Oxygen::QtSettings::loadKdeGlobals - kdeglobals: " << std::endl;
+        std::cerr << _kdeGlobals << std::endl;
+        #endif
+
+        // check change
+        return old != _kdeGlobals;
+
+    }
+
+    //_________________________________________________________
+    bool QtSettings::loadOxygen( void )
+    {
+
+        // save backup
+        OptionMap old = _oxygen;
+
+        // clear and reload
+        _oxygen.clear();
+        for( PathList::const_reverse_iterator iter = _kdeConfigPathList.rbegin(); iter != _kdeConfigPathList.rend(); ++iter )
+        {
+            const std::string filename( sanitizePath( *iter + "/oxygenrc" ) );
+            _oxygen.merge( filename );
+            monitorFile( filename );
+        }
+
+        #if OXYGEN_DEBUG
+        std::cerr << "Oxygen::QtSettings::loadOxygen - Oxygenrc: " << std::endl;
+        std::cerr << _oxygen << std::endl;
+        #endif
+
+        // check change
+        return old != _oxygen;
 
     }
 
@@ -222,11 +276,12 @@ namespace Oxygen
         PathList out;
 
         // load icon install prefix
-        char* path = 0L;
-        if( g_spawn_command_line_sync( "kde4-config --path config", &path, 0L, 0L, 0L ) && path )
+        gchar* path = 0L;
+        if( runCommand( "kde4-config --path config", path ) && path )
         {
 
             out.split( path );
+            g_free( path );
 
         } else {
 
@@ -252,8 +307,11 @@ namespace Oxygen
         // load icon install prefix
         PathList out;
         char* path = 0L;
-        if( g_spawn_command_line_sync( "kde4-config --path icon", &path, 0L, 0L, 0L ) && path )
-        { out.split( path ); }
+        if( runCommand( "kde4-config --path icon", path ) && path )
+        {
+            out.split( path );
+            g_free( path );
+        }
 
         // make sure defaultKdeIconPath is included in the list
         if( std::find( out.begin(), out.end(), _defaultKdeIconPath ) == out.end() )
@@ -273,7 +331,15 @@ namespace Oxygen
         // make sure that corresponding directory does exist
         struct stat st;
         if( stat( _userConfigDir.c_str(), &st ) != 0 )
-        { mkdir( _userConfigDir.c_str(), S_IRWXU|S_IRWXG|S_IRWXO ); }
+        {
+            #ifdef G_OS_WIN32
+            // S_IRWXG and S_IRWXO are undefined on Windows, and g_mkdir()
+            // ignores its second parameter on Windows anyway.
+            g_mkdir( _userConfigDir.c_str(), 0 );
+            #else
+            g_mkdir( _userConfigDir.c_str(), S_IRWXU|S_IRWXG|S_IRWXO );
+            #endif
+        }
 
         // note: in some cases, the target might exist and not be a directory
         // nothing we can do about it. We won't overwrite the file to prevent dataloss
@@ -623,6 +689,7 @@ namespace Oxygen
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[PRELIGHT]", _palette.color( Palette::Text ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[SELECTED]", _palette.color( Palette::SelectedText ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Text ) ) );
+        addLinkColors( "[Colors:Window]" );
 
         // buttons
         _rc.addSection( "oxygen-buttons-internal", Gtk::RC::defaultSection() );
@@ -634,6 +701,8 @@ namespace Oxygen
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[ACTIVE]", _palette.color( Palette::ButtonText ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[PRELIGHT]", _palette.color( Palette::ButtonText ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::ButtonText ) ) );
+        addLinkColors( "[Colors:Button]" );
+
         _rc.matchClassToSection( "*Button", "oxygen-buttons-internal" );
         _rc.matchClassToSection( "GtkOptionMenu", "oxygen-buttons-internal" );
         _rc.matchWidgetClassToSection( "*<GtkButton>.<GtkLabel>", "oxygen-buttons-internal" );
@@ -705,6 +774,7 @@ namespace Oxygen
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[NORMAL]", _palette.color( Palette::Base ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Base ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  base[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Base ) ) );
+        addLinkColors( "[Colors:View]" );
         _rc.matchClassToSection( "GtkSpinButton", "oxygen-entry-internal" );
         _rc.matchClassToSection( "GtkEntry", "oxygen-entry-internal" );
         _rc.matchClassToSection( "GtkTextView", "oxygen-entry-internal" );
@@ -717,6 +787,8 @@ namespace Oxygen
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[NORMAL]", _palette.color( Palette::TooltipText ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<int>( "  xthickness", 3 ) );
         _rc.addToCurrentSection( Gtk::RCOption<int>( "  ythickness", 3 ) );
+        addLinkColors( "[Colors:Tooltip]" );
+
         _rc.matchWidgetToSection( "gtk-tooltip*", "oxygen-tooltips-internal" );
 
         // special case for google chrome
@@ -728,6 +800,33 @@ namespace Oxygen
 
     }
 
+
+    //_________________________________________________________
+    void QtSettings::addLinkColors( const std::string& section )
+    {
+
+        {
+            // link colors
+            // from http://ubuntuforums.org/showthread.php?p=7009302
+            const ColorUtils::Rgba linkColor( ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( section, "ForegroundLink" ) ) );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  GtkWidget::link-color", linkColor ) );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  GtkHTML::alink_color", linkColor ) );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  GtkHTML::link_color", linkColor ) );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  GnomeHref::link-color", linkColor ) );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  GtkIMHtml::hyperlink-color", linkColor ) );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  GtkIMHtml::hyperlink-prelight-color", linkColor ) );
+        }
+
+        {
+            // visited link colors
+            // from http://ubuntuforums.org/showthread.php?p=7009302
+            const ColorUtils::Rgba visitedLinkColor( ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( section, "ForegroundVisited" ) ) );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  GtkWidget::visited-link-color", visitedLinkColor ) );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  GtkHTML::vlink_color", visitedLinkColor ) );
+        }
+
+    }
+
     //_________________________________________________________
     void QtSettings::loadKdeFonts( void )
     {
@@ -844,98 +943,98 @@ namespace Oxygen
     void QtSettings::loadOxygenOptions( void )
     {
 
-        OptionMap oxygen;
-        for( PathList::const_reverse_iterator iter = _kdeConfigPathList.rbegin(); iter != _kdeConfigPathList.rend(); ++iter )
-        { oxygen.merge( OptionMap( sanitizePath( *iter + "/oxygenrc" ) ) ); }
-
-        #if OXYGEN_DEBUG
-        std::cerr << "Oxygen::QtSettings::loadOxygenOptions - Oxygenrc: " << std::endl;
-        std::cerr << oxygen << std::endl;
-        #endif
-
         // background pixmap
-        _backgroundPixmap = oxygen.getValue( "[Common]", "BackgroundPixmap", "" );
+        _backgroundPixmap = _oxygen.getValue( "[Common]", "BackgroundPixmap", "" );
+
+        // background gradient
+        _useBackgroundGradient = ( _oxygen.getValue( "[Common]", "UseBackgroundGradient", "true" ) == "true" );
 
         // checkbox style
-        _checkBoxStyle = (oxygen.getValue( "[Style]", "CheckBoxStyle", "CS_CHECK" ) == "CS_CHECK") ? CS_CHECK:CS_X;
+        _checkBoxStyle = (_oxygen.getValue( "[Style]", "CheckBoxStyle", "CS_CHECK" ) == "CS_CHECK") ? CS_CHECK:CS_X;
 
         // checkbox style
-        _tabStyle = (oxygen.getValue( "[Style]", "TabStyle", "TS_SINGLE" ) == "TS_SINGLE") ? TS_SINGLE:TS_PLAIN;
+        _tabStyle = (_oxygen.getValue( "[Style]", "TabStyle", "TS_SINGLE" ) == "TS_SINGLE") ? TS_SINGLE:TS_PLAIN;
 
         // scrollbar buttons
-        _scrollBarAddLineButtons = oxygen.getOption( "[Style]", "ScrollBarAddLineButtons" ).toVariant<int>( 2 );
-        _scrollBarSubLineButtons = oxygen.getOption( "[Style]", "ScrollBarSubLineButtons" ).toVariant<int>( 1 );
+        _scrollBarAddLineButtons = _oxygen.getOption( "[Style]", "ScrollBarAddLineButtons" ).toVariant<int>( 2 );
+        _scrollBarSubLineButtons = _oxygen.getOption( "[Style]", "ScrollBarSubLineButtons" ).toVariant<int>( 1 );
 
         // toolbar separators
-        _toolBarDrawItemSeparator = oxygen.getOption( "[Style]", "ToolBarDrawItemSeparator" ).toVariant<std::string>("true") == "true";
+        _toolBarDrawItemSeparator = _oxygen.getOption( "[Style]", "ToolBarDrawItemSeparator" ).toVariant<std::string>("true") == "true";
 
         // tooltips
-        _tooltipTransparent = oxygen.getOption( "[Style]", "ToolTipTransparent" ).toVariant<std::string>("true") == "true";
-        _tooltipDrawStyledFrames = oxygen.getOption( "[Style]", "ToolTipDrawStyledFrames" ).toVariant<std::string>("true") == "true";
+        _tooltipTransparent = _oxygen.getOption( "[Style]", "ToolTipTransparent" ).toVariant<std::string>("true") == "true";
+        _tooltipDrawStyledFrames = _oxygen.getOption( "[Style]", "ToolTipDrawStyledFrames" ).toVariant<std::string>("true") == "true";
 
         // focus indicator in views
-        _viewDrawFocusIndicator = oxygen.getOption( "[Style]", "ViewDrawFocusIndicator" ).toVariant<std::string>("true") == "true";
+        _viewDrawFocusIndicator = _oxygen.getOption( "[Style]", "ViewDrawFocusIndicator" ).toVariant<std::string>("true") == "true";
 
         // tree branch lines
-        _viewDrawTreeBranchLines = oxygen.getOption( "[Style]", "ViewDrawTreeBranchLines" ).toVariant<std::string>("true") == "true";
+        _viewDrawTreeBranchLines = _oxygen.getOption( "[Style]", "ViewDrawTreeBranchLines" ).toVariant<std::string>("true") == "true";
 
         // triangular expanders
-        _viewDrawTriangularExpander = oxygen.getOption( "[Style]", "ViewDrawTriangularExpander" ).toVariant<std::string>("true") == "true";
+        _viewDrawTriangularExpander = _oxygen.getOption( "[Style]", "ViewDrawTriangularExpander" ).toVariant<std::string>("true") == "true";
 
         // triangular expander (arrow) size
-        std::string expanderSize( oxygen.getOption( "[Style]", "ViewTriangularExpanderSize" ).toVariant<std::string>("TE_SMALL") );
+        std::string expanderSize( _oxygen.getOption( "[Style]", "ViewTriangularExpanderSize" ).toVariant<std::string>("TE_SMALL") );
         if( expanderSize == "TE_NORMAL" ) _viewTriangularExpanderSize = ArrowNormal;
         else if( expanderSize == "TE_TINY" ) _viewTriangularExpanderSize = ArrowTiny;
         else _viewTriangularExpanderSize = ArrowSmall;
 
+        // invert view sort indicators
+        _viewInvertSortIndicator = _oxygen.getOption( "[Style]", "ViewInvertSortIndicator" ).toVariant<std::string>("false") == "true";
+
         // menu highlight mode
-        std::string highlightMode( oxygen.getOption( "[Style]", "MenuHighlightMode" ).toVariant<std::string>("MM_DARK") );
+        std::string highlightMode( _oxygen.getOption( "[Style]", "MenuHighlightMode" ).toVariant<std::string>("MM_DARK") );
         if( highlightMode == "MM_SUBTLE" ) _menuHighlightMode = MM_SUBTLE;
         else if( highlightMode == "MM_STRONG" ) _menuHighlightMode = MM_STRONG;
         else _menuHighlightMode = MM_DARK;
 
         // window drag mode
-        _windowDragEnabled = oxygen.getOption( "[Style]", "WindowDragEnabled" ).toVariant<std::string>("true") == "true";
+        _windowDragEnabled = _oxygen.getOption( "[Style]", "WindowDragEnabled" ).toVariant<std::string>("true") == "true";
 
-        std::string windowDragMode( oxygen.getOption( "[Style]", "WindowDragMode" ).toVariant<std::string>("WD_FULL") );
+        std::string windowDragMode( _oxygen.getOption( "[Style]", "WindowDragMode" ).toVariant<std::string>("WD_FULL") );
         if( windowDragMode == "WD_MINIMAL" ) _windowDragMode = WD_MINIMAL;
         else _windowDragMode = WD_FULL;
 
+        // use window manager to handle window drag
+        _useWMMoveResize = _oxygen.getOption( "[Style]", "UseWMMoveResize" ).toVariant<std::string>("true") == "true";
+
         // animations
-        _animationsEnabled = ( oxygen.getOption( "[Style]", "AnimationsEnabled" ).toVariant<std::string>("true") == "true" );
-        _genericAnimationsEnabled = ( oxygen.getOption( "[Style]", "GenericAnimationsEnabled" ).toVariant<std::string>("true") == "true" );
+        _animationsEnabled = ( _oxygen.getOption( "[Style]", "AnimationsEnabled" ).toVariant<std::string>("true") == "true" );
+        _genericAnimationsEnabled = ( _oxygen.getOption( "[Style]", "GenericAnimationsEnabled" ).toVariant<std::string>("true") == "true" );
 
         // menubar animation type
-        std::string menuBarAnimationType( oxygen.getValue( "[Style]", "MenuBarAnimationType", "MB_FADE") );
+        std::string menuBarAnimationType( _oxygen.getValue( "[Style]", "MenuBarAnimationType", "MB_FADE") );
         if( menuBarAnimationType == "MB_NONE" ) _menuBarAnimationType = None;
         else if( menuBarAnimationType == "MB_FADE" ) _menuBarAnimationType = Fade;
         else if( menuBarAnimationType == "MB_FOLLOW_MOUSE" ) _menuBarAnimationType = FollowMouse;
 
         // menubar animation type
-        std::string menuAnimationType( oxygen.getValue( "[Style]", "MenuAnimationType", "ME_FADE") );
+        std::string menuAnimationType( _oxygen.getValue( "[Style]", "MenuAnimationType", "ME_FADE") );
         if( menuAnimationType == "ME_NONE" ) _menuAnimationType = None;
         else if( menuAnimationType == "ME_FADE" ) _menuAnimationType = Fade;
         else if( menuAnimationType == "ME_FOLLOW_MOUSE" ) _menuAnimationType = FollowMouse;
 
         // toolbar animation type
-        std::string toolBarAnimationType( oxygen.getValue( "[Style]", "ToolBarAnimationType", "TB_FADE") );
+        std::string toolBarAnimationType( _oxygen.getValue( "[Style]", "ToolBarAnimationType", "TB_FADE") );
         if( toolBarAnimationType == "TB_NONE" ) _toolBarAnimationType = None;
         else if( toolBarAnimationType == "TB_FADE" ) _toolBarAnimationType = Fade;
         else if( toolBarAnimationType == "TB_FOLLOW_MOUSE" ) _toolBarAnimationType = FollowMouse;
 
         // animations duration
-        _genericAnimationsDuration = oxygen.getOption( "[Style]", "GenericAnimationsDuration" ).toVariant<int>(150);
-        _menuBarAnimationsDuration = oxygen.getOption( "[Style]", "MenuBarAnimationsDuration" ).toVariant<int>(150);
-        _menuBarFollowMouseAnimationsDuration = oxygen.getOption( "[Style]", "MenuBarFollowMouseAnimationsDuration" ).toVariant<int>(80);
-        _menuAnimationsDuration = oxygen.getOption( "[Style]", "MenuAnimationsDuration" ).toVariant<int>(150);
-        _menuFollowMouseAnimationsDuration = oxygen.getOption( "[Style]", "MenuFollowMouseAnimationsDuration" ).toVariant<int>(40);
-        _toolBarAnimationsDuration = oxygen.getOption( "[Style]", "ToolBarAnimationsDuration" ).toVariant<int>(50);
+        _genericAnimationsDuration = _oxygen.getOption( "[Style]", "GenericAnimationsDuration" ).toVariant<int>(150);
+        _menuBarAnimationsDuration = _oxygen.getOption( "[Style]", "MenuBarAnimationsDuration" ).toVariant<int>(150);
+        _menuBarFollowMouseAnimationsDuration = _oxygen.getOption( "[Style]", "MenuBarFollowMouseAnimationsDuration" ).toVariant<int>(80);
+        _menuAnimationsDuration = _oxygen.getOption( "[Style]", "MenuAnimationsDuration" ).toVariant<int>(150);
+        _menuFollowMouseAnimationsDuration = _oxygen.getOption( "[Style]", "MenuFollowMouseAnimationsDuration" ).toVariant<int>(40);
+        _toolBarAnimationsDuration = _oxygen.getOption( "[Style]", "ToolBarAnimationsDuration" ).toVariant<int>(50);
 
         // animation steps
-        TimeLine::setSteps( oxygen.getOption( "[Style]", "AnimationSteps" ).toVariant<int>( 10 ) );
+        TimeLine::setSteps( _oxygen.getOption( "[Style]", "AnimationSteps" ).toVariant<int>( 10 ) );
 
         // window decoration button size
-        std::string buttonSize( oxygen.getValue( "[Windeco]", "ButtonSize", "Normal") );
+        std::string buttonSize( _oxygen.getValue( "[Windeco]", "ButtonSize", "Normal") );
         if( buttonSize == "Small" ) _buttonSize = ButtonSmall;
         else if( buttonSize == "Large" ) _buttonSize = ButtonLarge;
         else if( buttonSize == "Very Large" ) _buttonSize = ButtonVeryLarge;
@@ -943,7 +1042,7 @@ namespace Oxygen
         else _buttonSize = ButtonDefault;
 
         // window decoration frame border size
-        std::string frameBorder(  oxygen.getValue( "[Windeco]", "FrameBorder", "Normal") );
+        std::string frameBorder(  _oxygen.getValue( "[Windeco]", "FrameBorder", "Normal") );
         if( frameBorder == "No Border" ) _frameBorder = BorderNone;
         else if( frameBorder == "No Side Border" ) _frameBorder = BorderNoSide;
         else if( frameBorder == "Tiny" ) _frameBorder = BorderTiny;
@@ -955,22 +1054,27 @@ namespace Oxygen
         else _frameBorder = BorderDefault;
 
         // window decoration title alignment
-        std::string titleAlign( oxygen.getValue( "[Windeco]", "TitleAlignment", "Center" ) );
+        std::string titleAlign( _oxygen.getValue( "[Windeco]", "TitleAlignment", "Center" ) );
         if( titleAlign == "Left" ) _titleAlignment = PANGO_ALIGN_LEFT;
         else if( titleAlign == "Center" ) _titleAlignment = PANGO_ALIGN_CENTER;
         else if( titleAlign == "Right" ) _titleAlignment = PANGO_ALIGN_RIGHT;
         else _titleAlignment = PANGO_ALIGN_CENTER;
 
         // Windeco radial gradient enable option
-        std::string wdBlendType( oxygen.getValue( "[Windeco]", "BlendColor", "Follow Style Hint" ) );
+        std::string wdBlendType( _oxygen.getValue( "[Windeco]", "BlendColor", "Follow Style Hint" ) );
         if( wdBlendType == "Follow Style Hint" ) _windecoBlendType=FollowStyleHint;
         else if( wdBlendType == "Radial Gradient" ) _windecoBlendType=RadialGradient;
         else if( wdBlendType == "Solid Color" ) _windecoBlendType=SolidColor;
         else _windecoBlendType=FollowStyleHint;
 
         // shadow configurations
-        _activeShadowConfiguration.initialize( oxygen );
-        _inactiveShadowConfiguration.initialize( oxygen );
+        _activeShadowConfiguration.initialize( _oxygen );
+        _inactiveShadowConfiguration.initialize( _oxygen );
+
+        if(_kdeGlobals.getOption( "[General]", "widgetStyle" ).toVariant<std::string>("oxygen") == "oxygen transparent")
+            _backgroundOpacity = _oxygen.getOption( "[Common]", "BackgroundOpacity" ).toVariant<int>(255);
+        else
+            _backgroundOpacity = 255;
 
         #if OXYGEN_DEBUG
         std::cerr << _activeShadowConfiguration << std::endl;
@@ -982,7 +1086,7 @@ namespace Oxygen
         _rc.setCurrentSection( Gtk::RC::defaultSection() );
         _rc.addToCurrentSection( Gtk::RCOption<int>(
             "  GtkScrollbar::slider-width",
-            oxygen.getOption( "[Style]", "ScrollBarWidth" ).toVariant<int>(15) + 1 ) );
+            _oxygen.getOption( "[Style]", "ScrollBarWidth" ).toVariant<int>(15) + 1 ) );
 
         _rc.addToCurrentSection( Gtk::RCOption<bool>("  GtkScrollbar::has-backward-stepper", _scrollBarSubLineButtons > 0 ) );
         _rc.addToCurrentSection( Gtk::RCOption<bool>("  GtkScrollbar::has-forward-stepper", _scrollBarAddLineButtons > 0 ) );
@@ -993,10 +1097,10 @@ namespace Oxygen
 
         // mnemonics
         GtkSettings* settings( gtk_settings_get_default() );
-        if( oxygen.hasOption( "[Style]", "MnemonicsMode" ) )
+        if( _oxygen.hasOption( "[Style]", "MnemonicsMode" ) )
         {
 
-            const std::string mnemonicsMode( oxygen.getOption( "[Style]", "MnemonicsMode" ).toVariant<std::string>("MN_ALWAYS") );
+            const std::string mnemonicsMode( _oxygen.getOption( "[Style]", "MnemonicsMode" ).toVariant<std::string>("MN_ALWAYS") );
             if( mnemonicsMode == "MN_NEVER" )
             {
 
@@ -1018,7 +1122,7 @@ namespace Oxygen
         } else {
 
             // for backward compatibility
-            const bool showMnemonics( oxygen.getOption( "[Style]", "ShowMnemonics" ).toVariant<std::string>("true") == "true" );
+            const bool showMnemonics( _oxygen.getOption( "[Style]", "ShowMnemonics" ).toVariant<std::string>("true") == "true" );
             if( showMnemonics )
             {
 
@@ -1036,6 +1140,42 @@ namespace Oxygen
 
     }
 
+    //_________________________________________________________
+    void QtSettings::loadExtraOptions( void )
+    {
+
+        // deal with pathbar button margins
+        // this needs to be done programatically in order to properly account for RTL locales
+        _rc.addSection( "oxygen-pathbutton-internal", Gtk::RC::defaultSection() );
+        _rc.addToCurrentSection( "  GtkButton::inner-border = { 2, 2, 1, 0 }" );
+
+        if( gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL )
+        {
+
+            _rc.addToCurrentSection( "  GtkToggleButton::inner-border={ 10, 0, 1, 0 }" );
+
+        } else {
+
+            _rc.addToCurrentSection( "  GtkToggleButton::inner-border={ 0, 10, 1, 0 }" );
+
+        }
+
+        _rc.matchWidgetClassToSection( "*PathBar.GtkToggleButton", "oxygen-pathbutton-internal" );
+
+        // entry margins
+        _rc.addSection( "oxygen-entry-margins-internal", Gtk::RC::defaultSection() );
+        _rc.addToCurrentSection( Gtk::RCOption<int>( "  xthickness", 5 ) );
+        _rc.addToCurrentSection( Gtk::RCOption<int>( "  ythickness", applicationName().isXul() ? 2:1 ) );
+        _rc.matchClassToSection( "GtkEntry", "oxygen-entry-margins-internal" );
+
+        // combobox buttons
+        _rc.addSection( "oxygen-combobox-button-internal", Gtk::RC::defaultSection() );
+        _rc.addToCurrentSection( Gtk::RCOption<int>( "  xthickness", 2 ) );
+        _rc.addToCurrentSection( Gtk::RCOption<int>( "  ythickness", applicationName().isXul() ? 2:0 ) );
+        _rc.matchWidgetClassToSection( "*<GtkComboBox>.<GtkButton>", "oxygen-combobox-button-internal" );
+
+    }
+
     //_________________________________________________________
     std::string QtSettings::sanitizePath( const std::string& path ) const
     {
@@ -1048,4 +1188,48 @@ namespace Oxygen
         return out;
     }
 
+    //_________________________________________________________
+    void QtSettings::monitorFile( const std::string& filename )
+    {
+
+        // check if file was already added
+        if( _monitoredFiles.find( filename ) != _monitoredFiles.end() )
+        { return; }
+
+        // check file existence
+        if( !std::ifstream( filename.c_str() ) )
+        { return; }
+
+        // create FileMonitor
+        FileMonitor monitor;
+        monitor.file = g_file_new_for_path( filename.c_str() );
+        if( ( monitor.monitor = g_file_monitor( monitor.file, G_FILE_MONITOR_NONE, 0L, 0L ) ) )
+        {
+
+            // insert in map
+            _monitoredFiles.insert( std::make_pair( filename, monitor ) );
+
+        } else {
+
+            // clear file and return
+            g_object_unref( monitor.file );
+            return;
+
+        }
+
+    }
+
+    //_________________________________________________________
+    void QtSettings::clearMonitoredFiles( void )
+    {
+        for( FileMap::iterator iter = _monitoredFiles.begin(); iter != _monitoredFiles.end(); iter++ )
+        {
+            iter->second.signal.disconnect();
+            g_object_unref( iter->second.file );
+            g_object_unref( iter->second.monitor );
+        }
+
+        _monitoredFiles.clear();
+    }
+
 }
